@@ -18,6 +18,8 @@ L<Jifty::Record> subclass that this action should update.
 
 use base qw/Jifty::Action::Record/;
 
+use Scalar::Util qw/ blessed /;
+
 =head1 METHODS
 
 =head2 arguments
@@ -125,7 +127,7 @@ sub take_action {
         # Skip fields that have not changed
         my $old = $self->record->$field;
         # XXX TODO: This ignore "by" on columns
-        $old = $old->id if ref($old) and $old->isa( 'Jifty::Record' );
+        $old = $old->id if blessed($old) and $old->isa( 'Jifty::Record' );
     
         # if both the new and old values are defined and equal, we don't want to change em
         # XXX TODO "$old" is a cheap hack to scalarize datetime objects
