@@ -13,11 +13,11 @@ Adds dispatcher rules for C</__jifty/js/*> and C</__jifty/css/*/>,
 which serve out compiled and compressed CSS and Javascript rules.
 
 =cut
-use Compress::Zlib qw();
 use HTTP::Date ();
 
 use Jifty::Dispatcher -base;
 
+
 on '/__jifty/js/*' => run {
     my $arg = $1;
     if ( $arg !~ /^[0-9a-f]{32}\.js$/ ) {
@@ -47,12 +47,13 @@ on '/__jifty/js/*' => run {
     # XXX TODO: If we start caching the squished JS in a file somewhere, we
     # can have the static handler serve it, which would take care of gzipping
     # for us.
-    if ( Jifty::View::Static::Handler->client_accepts_gzipped_content ) {
+    if ( $ccjs->gzip_enabled && Jifty::View::Static::Handler->client_accepts_gzipped_content ) {
         Jifty->log->debug("Sending gzipped squished JS");
         Jifty->handler->apache->header_out( "Content-Encoding" => "gzip" );
         Jifty->handler->apache->send_http_header();
         binmode STDOUT;
-        print Compress::Zlib::memGzip( $ccjs->cached_javascript );
+        print $ccjs->cached_javascript_gzip;
+
     } else {
         Jifty->log->debug("Sending squished JS");
         Jifty->handler->apache->send_http_header();
@@ -63,17 +64,18 @@ on '/__jifty/js/*' => run {
 
 on '/__jifty/css/*' => run {
     my $arg = $1;
-    if ( $arg !~ /^[0-9a-f]{32}\.css$/ ) {
+    my ($ccjs) = Jifty->find_plugin('Jifty::Plugin::CompressedCSSandJS');
+    if ( $arg !~ /^[0-9a-f]{32}\.css$/ || !$ccjs) {
 
         # This doesn't look like a real request for squished CSS,
         # so redirect to a more failsafe place
         Jifty->web->redirect( "/static/css/" . $arg );
     }
 
-    Jifty->web->generate_css;
+    $ccjs->generate_css;
 
     if ( Jifty->handler->cgi->http('If-Modified-Since')
-        and $arg eq Jifty->web->cached_css_digest . '.css' )
+        and $arg eq $ccjs->cached_css_digest . '.css' )
     {
         Jifty->log->debug("Returning 304 for cached css");
         Jifty->handler->apache->header_out( Status => 304 );
@@ -86,16 +88,16 @@ on '/__jifty/css/*' => run {
     # XXX TODO: If we start caching the squished CSS in a file somewhere, we
     # can have the static handler serve it, which would take care of gzipping
     # for us.
-    if ( Jifty::View::Static::Handler->client_accepts_gzipped_content ) {
+    if ($ccjs->gzip_enabled && Jifty::View::Static::Handler->client_accepts_gzipped_content ) {
         Jifty->log->debug("Sending gzipped squished CSS");
         Jifty->handler->apache->header_out( "Content-Encoding" => "gzip" );
         Jifty->handler->apache->send_http_header();
         binmode STDOUT;
-        print Compress::Zlib::memGzip( Jifty->web->cached_css );
+        print $ccjs->cached_css_gzip;
     } else {
         Jifty->log->debug("Sending squished CSS");
         Jifty->handler->apache->send_http_header();
-        print Jifty->web->cached_css;
+        print $ccjs->cached_css;
     }
     abort;
 };
