@@ -17,20 +17,20 @@ Jifty.web = function() { return Jifty.Web };
 function _get_named_args(args) {
     var result = {};
     for (var i = 0; i < args.length; i+=2) {
-	result[args[i]] = args[i+1];
+        result[args[i]] = args[i+1];
     }
     return result;
 
 }
 
 function _get_onclick(action_hash, name, args, path) {
-    var onclick = 'if(event.ctrlKey||event.metaKey||event.altKey||event.shiftKey) return true; return update('
+    var onclick = 'if(event.ctrlKey||event.metaKey||event.altKey||event.shiftKey) return true; return Jifty.update('
     + JSON.stringify({'continuation': {},
-		      'actions': action_hash,
-		      'fragments': [{'mode': 'Replace', 'args': args, 'region': name, 'path': path}]})
+                      'actions': action_hash,
+                      'fragments': [{'mode': 'Replace', 'args': args, 'region': name, 'path': path}]})
     +', this)';
     onclick = onclick.replace(/"/g, "'"); //"' )# grr emacs!
-	return onclick;
+        return onclick;
 }
 // XXX
 var hyperlink  = function() {
@@ -38,9 +38,9 @@ var hyperlink  = function() {
     var current_region = Jifty.Web.current_region;
     var onclick = _get_onclick({}, current_region.name, current_region.args, args.onclick[0].replace_with);
     outs( a(function() { attr(function()
-			      {return ['onclick', onclick, 'href', '#']});
-	    return args.label
-		}));
+                              {return ['onclick', onclick, 'href', '#']});
+            return args.label
+                }));
 }
 
 var render_param = function(a, field) { outs(a.render_param(field)) };
@@ -57,27 +57,27 @@ var form_return  = function() {
     var current_region = Jifty.Web.current_region;
     var onclick = _get_onclick(action_hash, current_region.name, current_region.args, current_region.path);
     outs(
-	 div(function() {
-		 attr(function() { return ['class', 'submit_button'] });
-		 return input(function() { attr(function()
-						{return ['type', 'submit',
-							 'onclick', onclick,
-							 'class', 'widget button',
-							 'id', 'S' + (++SERIAL + SERIAL_postfix),
-							 'value', args.label,
-							 'name', 'J:V-region-__page-signup_widget=_signup|J:ACTIONS=signupnow'] })});
-		     }));
+         div(function() {
+                 attr(function() { return ['class', 'submit_button'] });
+                 return input(function() { attr(function()
+                                                {return ['type', 'submit',
+                                                         'onclick', onclick,
+                                                         'class', 'widget button',
+                                                         'id', 'S' + (++SERIAL + SERIAL_postfix),
+                                                         'value', args.label,
+                                                         'name', 'J:V-region-__page-signup_widget=_signup|J:ACTIONS=signupnow'] })});
+                     }));
 
 };
 
 function register_action(a) {
     outs(div(function() {
-		attr(function() { return ['class', 'hidden'] });
-		return input(function() { attr(function() {
-				return ['type', 'hidden',
-					'name', a.register_name(),
-					'id', a.register_name(),
-					'value', a.actionClass] }) } ) } ));
+                attr(function() { return ['class', 'hidden'] });
+                return input(function() { attr(function() {
+                                return ['type', 'hidden',
+                                        'name', a.register_name(),
+                                        'id', a.register_name(),
+                                        'value', a.actionClass] }) } ) } ));
     /* XXX: fallback values */
 }
 
@@ -335,23 +335,23 @@ Action.prototype = {
 
     /* client side logic extracted from Jifty::Action */
     _action_spec: function() {
-	if (!this.s_a) {
-	    /* XXX: make REST client accessible */
-	    var Todo = new AsynapseRecord('todo');
-	    this.s_a = $H(Todo.eval_ajax_get('/=/action/'+this.actionClass+'.js'));
-	}
-	
-	return this.s_a
+        if (!this.s_a) {
+            /* XXX: make REST client accessible */
+            var Todo = new AsynapseRecord('todo');
+            this.s_a = $H(Todo.eval_ajax_get('/=/action/'+this.actionClass+'.js'));
+        }
+        
+        return this.s_a
     },
     argument_names: function() {
-	return this._action_spec().keys();
+        return this._action_spec().keys();
     },
 
     render_param: function(field) {
-	var a_s = this._action_spec();
-	var type = 'text';
-	var f = new ActionField(field, a_s[field], this);
-	return f.render();
+        var a_s = this._action_spec();
+        var type = 'text';
+        var f = new ActionField(field, a_s[field], this);
+        return f.render();
     },
     register_name: function() { return this.register.id }
 
@@ -362,110 +362,110 @@ var SERIAL = 0;
 ActionField = Class.create();
 ActionField.prototype = {
  initialize: function(name, args, action) {
-	this.name = name;
-	this.label = args.label;
-	this.hints = args.hints;
-	this.mandatory = args.mandatory;
-	this.ajax_validates = args.ajax_validates;
-	this.current_value = action.data_structure().fields[name].value;
+        this.name = name;
+        this.label = args.label;
+        this.hints = args.hints;
+        this.mandatory = args.mandatory;
+        this.ajax_validates = args.ajax_validates;
+        this.current_value = action.data_structure().fields[name].value;
         this.error = action.result.field_error[name];
-	this.action = action;
-	if (!this.render_mode) this.render_mode = 'update';
-	this.type = 'text';
+        this.action = action;
+        if (!this.render_mode) this.render_mode = 'update';
+        this.type = 'text';
     },
 
  render: function() {
-	if (this.render_mode == 'read')
-	    return this.render_wrapper
-		(this.render_preamble,
-		 this.render_label,
-		 this.render_value);
-	else
-	    return this.render_wrapper
-	    (this.render_preamble,
-	     this.render_label,
-	     this.render_widget,
-	     this.render_autocomplete_div,
-	     this.render_inline_javascript,
-	     this.render_hints,
-	     this.render_errors,
-	     this.render_warnings,
-	     this.render_canonicalization_notes);
+        if (this.render_mode == 'read')
+            return this.render_wrapper
+                (this.render_preamble,
+                 this.render_label,
+                 this.render_value);
+        else
+            return this.render_wrapper
+            (this.render_preamble,
+             this.render_label,
+             this.render_widget,
+             this.render_autocomplete_div,
+             this.render_inline_javascript,
+             this.render_hints,
+             this.render_errors,
+             this.render_warnings,
+             this.render_canonicalization_notes);
     },
  render_wrapper: function () {
-	var classes = ['form_field'];
-	if (this.mandatory) classes.push('mandatory');
-	if (this.name) classes.push('argument-'+this.name);
-	var args = arguments;
-	var tthis = this;
-	return div(function() {
-		attr(function(){return ['class', classes.join(' ')]});
-		var buf = new Array;
-		for (var i = 0; i < args.length; ++i) {
-		    buf.push(typeof(args[i]) == 'function' ? args[i].apply(tthis) : args[i]);
-		}
-		return buf.join('');
-	    });
+        var classes = ['form_field'];
+        if (this.mandatory) classes.push('mandatory');
+        if (this.name) classes.push('argument-'+this.name);
+        var args = arguments;
+        var tthis = this;
+        return div(function() {
+                attr(function(){return ['class', classes.join(' ')]});
+                var buf = new Array;
+                for (var i = 0; i < args.length; ++i) {
+                    buf.push(typeof(args[i]) == 'function' ? args[i].apply(tthis) : args[i]);
+                }
+                return buf.join('');
+            });
     },
     render_preamble: function() {
-	var tthis = this;
-	return span(function(){attr(function(){return ['class', "preamble"]});
-		return tthis.preamble });
+        var tthis = this;
+        return span(function(){attr(function(){return ['class', "preamble"]});
+                return tthis.preamble });
     },
 
     render_label: function() {
-	var tthis = this;
-	if(this.render_mode == 'update')
-	    return label(function(){attr(function(){return['class', "label", 'for', tthis.element_id()]});
-		    return tthis.label });
-	else
-	    return span(function(){attr(function(){return['class', "label" ]});
-		    return tthis.label });
+        var tthis = this;
+        if(this.render_mode == 'update')
+            return label(function(){attr(function(){return['class', "label", 'for', tthis.element_id()]});
+                    return tthis.label });
+        else
+            return span(function(){attr(function(){return['class', "label" ]});
+                    return tthis.label });
     },
  input_name: function() {
-	return ['J:A:F', this.name, this.action.moniker].join('-');
+        return ['J:A:F', this.name, this.action.moniker].join('-');
     },
  render_hints: function() {
-	var tthis = this;
-	return span(function(){attr(function(){return ['class', "hints"]});
-		return tthis.hints });
+        var tthis = this;
+        return span(function(){attr(function(){return ['class', "hints"]});
+                return tthis.hints });
     },
 
  render_errors: function() {
-	if (!this.action) return '';
-	var tthis = this;
-	// XXX: post-request handler needs to extract field error messages
-	return span(function(){attr(function(){return ['class', "error", 'id', 'errors-'+tthis.input_name()]});
-		return tthis.error });
+        if (!this.action) return '';
+        var tthis = this;
+        // XXX: post-request handler needs to extract field error messages
+        return span(function(){attr(function(){return ['class', "error", 'id', 'errors-'+tthis.input_name()]});
+                return tthis.error });
     },
 
  render_widget: function () {
-	var tthis = this;
-	return input(function(){
-		    attr(function(){
-			    var fields = ['type', tthis.type];
-			    if (tthis.input_name) fields.push('name', tthis.input_name());
-			    fields.push('id', tthis.element_id());
-			    if (tthis.current_value) fields.push('value', tthis.current_value);
-			    fields.push('class', tthis._widget_class().join(' '));
-			    if (tthis.max_length) fields.push('size', tthis.max_length, 'maxlength', tthis.max_length);
-			    if (tthis.disable_autocomplete) fields.push('autocomplete', "off");
-			    //" " .$self->other_widget_properties;
-			    return fields;
-			})});
+        var tthis = this;
+        return input(function(){
+                    attr(function(){
+                            var fields = ['type', tthis.type];
+                            if (tthis.input_name) fields.push('name', tthis.input_name());
+                            fields.push('id', tthis.element_id());
+                            if (tthis.current_value) fields.push('value', tthis.current_value);
+                            fields.push('class', tthis._widget_class().join(' '));
+                            if (tthis.max_length) fields.push('size', tthis.max_length, 'maxlength', tthis.max_length);
+                            if (tthis.disable_autocomplete) fields.push('autocomplete', "off");
+                            //" " .$self->other_widget_properties;
+                            return fields;
+                        })});
     },
  _widget_class: function() {
-	var classes = ['form_field'];
-	if (this.mandatory)      classes.push('mandatory');
-	if (this.name)           classes.push('argument-'+this.name);
-	if (this.ajax_validates) classes.push('ajaxvalidation');
-	return classes;
+        var classes = ['form_field'];
+        if (this.mandatory)      classes.push('mandatory');
+        if (this.name)           classes.push('argument-'+this.name);
+        if (this.ajax_validates) classes.push('ajaxvalidation');
+        return classes;
     },
 
  element_id: function() { if(!this._element_id) this._element_id = this.input_name() + '-S' + (++SERIAL + SERIAL_postfix);
-			  return this._element_id; },
+                          return this._element_id; },
  __noSuchMethod__: function(name) {
-	return '<!-- '+name+' not implemented yet -->';
+        return '<!-- '+name+' not implemented yet -->';
     }
 
 };
@@ -519,9 +519,9 @@ Object.extend(Form.Element, {
     getAction: function (element) {
         element = $(element);    
         var moniker = Form.Element.getMoniker(element);
-	if (!current_actions[moniker])
-	    current_actions[moniker] = new Action(moniker);
-	return current_actions[moniker];
+        if (!current_actions.get(moniker))
+            current_actions.set(moniker, new Action(moniker));
+        return current_actions.get(moniker);
     },
 
     // Returns the name of the field
@@ -600,20 +600,23 @@ Object.extend(Form.Element, {
          && ((element.nodeName != 'A')     || (! element.getAttribute("name"))))
             return $H();
 
+        if (element.getAttribute("name").length == 0)
+            return $H();
+
         var extras = $H();
 
         // Split other arguments out, if we're on a button
         var pairs = element.getAttribute("name").split("|");
         for (var i = 0; i < pairs.length; i++) {
             var bits = pairs[i].split('=',2);
-            extras[bits[0]] = bits[1];
+            extras.set(bits[0], bits[1]);
         }
         return extras;
     },
 
     buttonActions: function(element) {
         element = $(element);
-        var actions = Form.Element.buttonArguments(element)['J:ACTIONS'];
+        var actions = Form.Element.buttonArguments(element).get('J:ACTIONS');
         if(actions) {
             return actions.split(",");
         } else {
@@ -631,7 +634,7 @@ Object.extend(Form.Element, {
             var e = document.createElement("input");
             e.setAttribute("type", "hidden");
             e.setAttribute("name", keys[i]);
-            e.setAttribute("value", args[keys[i]]);
+            e.setAttribute("value", args.get(keys[i]));
             e['virtualform'] = Form.Element.getForm(element);
             extras.push(e);
         }
@@ -734,8 +737,8 @@ Region.prototype = {
         this.name = name;
         this.args = $H(args);
         this.path = path;
-        this.parent = parent ? fragments[parent] : null;
-        if (fragments[name]) {
+        this.parent = parent ? fragments.get(parent) : null;
+        if (fragments.get(name)) {
             // If this fragment already existed, we want to wipe out
             // whatever evil lies we might have said earlier; do this
             // by clearing out everything that looks relevant
@@ -744,12 +747,12 @@ Region.prototype = {
                 var k = keys[i];
                 var parsed = k.match(/^(.*?)\.(.*)/);
                 if ((parsed != null) && (parsed.length == 3) && (parsed[1] == this.name)) {
-                    delete current_args[k];
+                    current_args.unset(k);
                 }
             }
         }
 
-        fragments[name] = this;
+        fragments.set(name, this);
     },
 
     setPath: function(supplied) {
@@ -758,7 +761,7 @@ Region.prototype = {
         for (var i = 0; i < keys.length; i++) {
             var k = keys[i];
             if (k == this.name) {
-                this.path = current_args[k];
+                this.path = current_args.get(k);
             }
         }
 
@@ -768,7 +771,7 @@ Region.prototype = {
         }
         
         // Propagate back to current args
-        current_args[this.name] = this.path;
+        current_args.set(this.name, this.path);
 
         // Return new value
         return this.path;
@@ -782,7 +785,7 @@ Region.prototype = {
             var k = keys[i];
             var parsed = k.match(/^(.*?)\.(.*)/);
             if ((parsed != null) && (parsed.length == 3) && (parsed[1] == this.name)) {
-                this.args[parsed[2]] = current_args[k];
+                this.args.set(parsed[2], current_args.get(k));
             }
         }
 
@@ -793,7 +796,7 @@ Region.prototype = {
         keys = supplied.keys();
         for (var i = 0; i < keys.length; i++) {
             var k = keys[i];
-            current_args[this.name+'.'+k] = supplied[k];
+            current_args.set(this.name+'.'+k, supplied.get(k));
         }
         
         // Return new values
@@ -851,12 +854,12 @@ function prepare_element_for_update(f) {
         // If we're removing the element, do it now
         // XXX TODO: Effects on this?
         if (f['mode'] == "Delete") {
-            fragments[name] = null;
+            fragments.set(name, null);
             Element.remove(element);
             return;
         }
 
-        f['is_new'] = (fragments[name] ? false : true);
+        f['is_new'] = (fragments.get(name) ? false : true);
         // If it's new, we need to create it so we can dump it
         if (f['is_new']) {
             // Find what region we're inside
@@ -875,14 +878,14 @@ function prepare_element_for_update(f) {
 
             // Make the region (for now)
             new Region(name, f['args'], f['path'], f['parent']);
-        } else if ((f['path'] != null) && f['toggle'] && (f['path'] == fragments[name].path)) {
+        } else if ((f['path'] != null) && f['toggle'] && (f['path'] == fragments.get(name).path)) {
             // If they set the 'toggle' flag, and clicking wouldn't change the path
             Element.update(element, '');
-            fragments[name].path = null;
+            fragments.get(name).path = null;
             return;
         } else if (f['path'] == null) {
             // If they didn't know the path, fill it in now
-            f['path'] == fragments[name].path;
+            f['path'] == fragments.get(name).path;
         }
 
     return f;    
@@ -897,7 +900,7 @@ var walk_node = function(node, table) {
          child = child.nextSibling) {
         var name = child.nodeName.toLowerCase();
         if (table[name])
-	    table[name](child);
+            table[name](child);
     }
 }
 
@@ -911,14 +914,14 @@ var extract_cacheable = function(fragment, f) {
             } else if (fragment_bit.firstChild) {
                 textContent = fragment_bit.firstChild.nodeValue;
             } 
-	    try {
-		var cache_func = eval(textContent);
-		CACHE[f['path']] = { 'type': c_type, 'content': cache_func };
-	    }
-	    catch(e) { 
-		alert(e);
-		alert(textContent);
-	    }
+            try {
+                var cache_func = eval(textContent);
+                CACHE[f['path']] = { 'type': c_type, 'content': cache_func };
+            }
+            catch(e) { 
+                alert(e);
+                alert(textContent);
+            }
         }
     });
 };
@@ -928,67 +931,69 @@ var extract_cacheable = function(fragment, f) {
 //   - f: fragment spec
 var apply_fragment_updates = function(fragment, f) {
     // We found the right fragment
-    var dom_fragment = fragments[f['region']];
+    var dom_fragment = fragments.get(f['region']);
     var new_dom_args = $H();
 
     var element = f['element'];
     walk_node(fragment,
     { argument: function(fragment_bit) {
-	    // First, update the fragment's arguments
-	    // with what the server actually used --
-	    // this is needed in case there was
-	    // argument mapping going on
-	    var textContent = '';
-	    if (fragment_bit.textContent) {
-		textContent = fragment_bit.textContent;
-	    } else if (fragment_bit.firstChild) {
-		textContent = fragment_bit.firstChild.nodeValue;
-	    }
-	    new_dom_args[fragment_bit.getAttribute("name")] = textContent;
-	},
+            // First, update the fragment's arguments
+            // with what the server actually used --
+            // this is needed in case there was
+            // argument mapping going on
+            var textContent = '';
+            if (fragment_bit.textContent) {
+                textContent = fragment_bit.textContent;
+            } else if (fragment_bit.firstChild) {
+                textContent = fragment_bit.firstChild.nodeValue;
+            }
+            new_dom_args.set(fragment_bit.getAttribute("name"), textContent);
+        },
       content: function(fragment_bit) {
-	    var textContent = '';
-	    if (fragment_bit.textContent) {
-		textContent = fragment_bit.textContent;
-	    } else if (fragment_bit.firstChild) {
-		textContent = fragment_bit.firstChild.nodeValue;
-	    }
+            var textContent = '';
+            if (fragment_bit.textContent) {
+                textContent = fragment_bit.textContent;
+            } else if (fragment_bit.firstChild) {
+                textContent = fragment_bit.firstChild.nodeValue;
+            }
                     
-	    // Once we find it, do the insertion
-	    if (f['mode'] && (f['mode'] != 'Replace')) {
-		var insertion = eval('Insertion.'+f['mode']);
-		new insertion(element, textContent.stripScripts());
-	    } else {
-		Element.update(element, textContent.stripScripts());
-	    }
-	    // We need to give the browser some "settle" time before
-	    // we eval scripts in the body
+            // Once we find it, do the insertion
+            if (f['mode'] && (f['mode'] != 'Replace')) {
+                var insertion = eval('Insertion.'+f['mode']);
+                new insertion(element, textContent.stripScripts());
+            } else {
+                Element.update(element, textContent.stripScripts());
+            }
+            // We need to give the browser some "settle" time before
+            // we eval scripts in the body
         YAHOO.util.Event.onAvailable(element.id, function() {
             (function() { this.evalScripts() }).bind(textContent)();
         });
         Behaviour.apply(element);
-	}
+        }
     });
     dom_fragment.setArgs(new_dom_args);
 
     // Also, set us up the effect
     if (f['effect']) {
-	try {
-	    var effect = eval('Effect.'+f['effect']);
-	    var effect_args  = f['effect_args'] || {};
-	    if (effect) {
-		if (f['is_new'])
-		    Element.hide($('region-'+f['region']));
-		(effect)($('region-'+f['region']), effect_args);
-	    }
-	} catch ( e ) {
-	    // Don't be sad if the effect doesn't exist
-	}
+        try {
+            var effect = eval('Effect.'+f['effect']);
+            var effect_args  = f['effect_args'] || {};
+            if (effect) {
+                if (f['is_new'])
+                    Element.hide($('region-'+f['region']));
+                (effect)($('region-'+f['region']), effect_args);
+            }
+        } catch ( e ) {
+            // Don't be sad if the effect doesn't exist
+        }
     }
 }
 
 // Update a region.  Takes a hash of named parameters, including:
 //  - 'actions' is an array of monikers to submit
+//  - 'action_arguments' is a hash of action monikers to hashes of arguments which should override any arguments coming from form fields
+//        the hash keys for 'action_arguments' are the values of the 'actions' array
 //  - 'fragments' is an array of hashes, which may have:
 //     - 'region' is the name of the region to update
 //     - 'args' is a hash of arguments to override
@@ -996,7 +1001,7 @@ var apply_fragment_updates = function(fragment, f) {
 //     - 'element' is the CSS selector of the element to update, if 'region' isn't supplied
 //     - 'mode' is one of 'Replace', or the name of a Prototype Insertion
 //     - 'effect' is the name of a Prototype Effect
-function update() {
+Jifty.update = function () {
     // loads
     if(!Ajax.getTransport()) return true;
     // XXX: prevent default behavior in IE
@@ -1014,7 +1019,7 @@ function update() {
     var disabled_elements = $A();
 
     // Set request base path
-    request['path'] = '/__jifty/webservices/xml';
+    request.set('path', '/__jifty/webservices/xml');
 
     // Grab extra arguments (from a button)
     var button_args = Form.Element.buttonFormElements(trigger);
@@ -1031,14 +1036,14 @@ function update() {
     }
     var optional_fragments;
     if (form && form['J:CALL']) 
-	optional_fragments = [ prepare_element_for_update({'mode':'Replace','args':{},'region':'__page','path': null}) ];
+        optional_fragments = [ prepare_element_for_update({'mode':'Replace','args':{},'region':'__page','path': null}) ];
     // Build actions structure
     var has_request = 0;
-    request['actions'] = $H();
+    request.set('actions', $H());
     for (var moniker in named_args['actions']) {
         var disable = named_args['actions'][moniker];
         var a = new Action(moniker, button_args);
-	current_actions[moniker] = a; // XXX: how do i make this bloody singleton?
+            current_actions.set(moniker, a); // XXX: how do i make this bloody singleton?
         // Special case for Redirect, allow optional, implicit __page
         // from the response to be used.
         if (a.actionClass == 'Jifty::Action::Redirect')
@@ -1050,13 +1055,24 @@ function update() {
             if(disable) {
                 a.disable_input_fields(disabled_elements);
             }
-            request['actions'][moniker] = a.data_structure();
+            var param = a.data_structure();
+            var fields = param.fields;
+            var override = named_args['action_arguments'][param.moniker] || {};
+            for (var argname in override) {
+                if (fields[argname]) {
+                    fields[argname].value = override[argname];
+                }
+                else {
+                    fields[argname] = { value: override[argname] };
+                }
+            }
+            request.get('actions').set(moniker, param);
             ++has_request;
         }
 
     }
 
-    request['fragments'] = $H();
+    request.set('fragments', $H());
     var update_from_cache = new Array;
 
     // Build fragments structure
@@ -1069,11 +1085,11 @@ function update() {
         if (cached && cached['type'] == 'static') {
             var my_fragment = document.createElement('fragment');
             var content_node = document.createElement('content');
-	    var cached_result;
+            var cached_result;
 
-	    Jifty.Web.current_region = fragments[f['region']];
-	    try { cached_result = apply_cached_for_action(cached['content'], []) }
-	    catch (e) { alert(e) }
+            Jifty.Web.current_region = fragments.get(f['region']);
+            try { cached_result = apply_cached_for_action(cached['content'], []) }
+            catch (e) { alert(e) }
 
             content_node.textContent = cached_result;
             my_fragment.appendChild(content_node);
@@ -1083,49 +1099,49 @@ function update() {
  } );
             continue;
         }
-	else if (cached && cached['type'] == 'action') {
+        else if (cached && cached['type'] == 'action') {
             var my_fragment = document.createElement('fragment');
             var content_node = document.createElement('content');
 
             my_fragment.appendChild(content_node);
             my_fragment.setAttribute('id', f['region']);
             update_from_cache.push(function(){
-		    var cached_result;
-		    Jifty.Web.current_region = fragments[f['region']];
-		    try {
-			cached_result = apply_cached_for_action(cached['content'], Form.getActions(form));
-		    }
-		    catch (e) { alert(e); throw e }
-		    content_node.textContent = cached_result;
-		    apply_fragment_updates(my_fragment, f);
+                    var cached_result;
+                    Jifty.Web.current_region = fragments.get(f['region']);
+                    try {
+                        cached_result = apply_cached_for_action(cached['content'], Form.getActions(form));
+                    }
+                    catch (e) { alert(e); throw e }
+                    content_node.textContent = cached_result;
+                    apply_fragment_updates(my_fragment, f);
  } );
             continue;
-	}
+        }
         else if (cached && cached['type'] == 'crudview') {
-	    try { // XXX: get model class etc as metadata in cache 
-		// XXX: kill dup code
-	    var Todo = new AsynapseRecord('todo');
-	    var record = Todo.find(f['args']['id']);
+            try { // XXX: get model class etc as metadata in cache 
+                // XXX: kill dup code
+            var Todo = new AsynapseRecord('todo');
+            var record = Todo.find(f['args']['id']);
             var my_fragment = document.createElement('fragment');
             var content_node = document.createElement('content');
             content_node.textContent = cached['content'](record);
             my_fragment.appendChild(content_node);
             my_fragment.setAttribute('id', f['region']);
             update_from_cache.push(function(){ apply_fragment_updates(my_fragment, f); } );
-	    } catch (e) { alert(e) };
-	    continue;
-	}
+            } catch (e) { alert(e) };
+            continue;
+        }
 
         // Update with all new values
         var name = f['region'];
-        var fragment_request = fragments[name].data_structure(f['path'], f['args']);
+        var fragment_request = fragments.get(name).data_structure(f['path'], f['args']);
 
         if (f['is_new'])
             // Ask for the wrapper if we are making a new region
             fragment_request['wrapper'] = 1;
 
         // Push it onto the request stack
-        request['fragments'][name] = fragment_request;
+        request.get('fragments').set(name, fragment_request);
         ++has_request;
     }
 
@@ -1142,28 +1158,28 @@ function update() {
         // Grab the XML response
         var response = transport.responseXML.documentElement;
 
-	// Get action results
+        // Get action results
         walk_node(response,
-	{ result: function(result) {
-		var moniker = result.getAttribute("moniker");
-		walk_node(result,
-			  { field: function(field) {
-				  var error = field.getElementsByTagName('error')[0];
-				  if (error) {
-				      var text = error.textContent
-					  ? error.textContent
-					  : (error.firstChild ? error.firstChild.nodeValue : '');
-				      var action = current_actions[moniker];
-				      action.result.field_error[field.getAttribute("name")] = text;
-				      }
-			      }});
-	    }});
+        { result: function(result) {
+                var moniker = result.getAttribute("moniker");
+                walk_node(result,
+                          { field: function(field) {
+                                  var error = field.getElementsByTagName('error')[0];
+                                  if (error) {
+                                      var text = error.textContent
+                                          ? error.textContent
+                                          : (error.firstChild ? error.firstChild.nodeValue : '');
+                                      var action = current_actions.get(moniker);
+                                      action.result.field_error[field.getAttribute("name")] = text;
+                                      }
+                              }});
+            }});
 
         for ( var i = 0; i < disabled_elements.length; i++ ) {
             disabled_elements[i].disabled = false;
         }
 
-	// empty known action. XXX: we should only need to discard actions being submitted
+        // empty known action. XXX: we should only need to discard actions being submitted
 
         // Loop through the result looking for it
         var expected_fragments = optional_fragments ? optional_fragments : named_args['fragments'];
@@ -1182,20 +1198,20 @@ function update() {
             extract_cacheable(response_fragment, f);
         }
 
-	update_from_cache.each(function(x) { x() });
+        update_from_cache.each(function(x) { x() });
 
         walk_node(response,
-	{ result: function(result) {
+        { result: function(result) {
                 for (var key = result.firstChild;
                      key != null;
                      key = key.nextSibling) {
                     show_action_result(result.getAttribute("moniker"),key);
                 }
             },
-	  redirect: function(redirect) {
+          redirect: function(redirect) {
                 document.location =  redirect.firstChild.firstChild.nodeValue;
-	}});
-	current_actions = $H();
+        }});
+        current_actions = $H();
     };
     var onFailure = function(transport, object) {
         hide_wait_message_now();
@@ -1210,15 +1226,15 @@ function update() {
     };
 
     // Build variable structure
-    request['variables'] = $H();
+    request.set('variables', $H());
     var keys = current_args.keys();
     for (var i = 0; i < keys.length; i++) {
         var k = keys[i];
-        request['variables']['region-'+k] = current_args[k];
+        request.get('variables').set('region-'+k, current_args.get(k));
     }
 
     // Build continuation structure
-    request['continuation'] = named_args['continuation'];
+    request.set('continuation', named_args['continuation']);
 
     // Push any state variables which we set into the forms
     for (var i = 0; i < document.forms.length; i++) {
@@ -1227,20 +1243,20 @@ function update() {
         for (var j = 0; j < keys.length; j++) {
             var n = keys[j];
             if (form['J:V-region-'+n]) {
-                form['J:V-region-'+n].value = current_args[n];
+                form['J:V-region-'+n].value = current_args.get(n);
             } else {
                 var hidden = document.createElement('input');
                 hidden.setAttribute('type',  'hidden');
                 hidden.setAttribute('name',  'J:V-region-'+n);
                 hidden.setAttribute('id',    'J:V-region-'+n);
-                hidden.setAttribute('value', current_args[n]);
+                hidden.setAttribute('value', current_args.get(n));
                 form.appendChild(hidden);
             }
         }
     }
 
     // Set up our options
-    var options = { postBody: JSON.stringify(request),
+    var options = { postBody: request.toJSON(), //JSON.stringify(request.toObject),
                     onSuccess: onSuccess,
                     onException: onFailure,
                     onFailure: onFailure,
@@ -1255,6 +1271,11 @@ function update() {
     return false;
 }
 
+function update ( named_args, trigger ) {
+    alert( 'please use Jifty.update instead of update.' );
+    return Jifty.update( named_args, trigger );
+}
+
 function trace( msg ){
   if( typeof( jsTrace ) != 'undefined' ){
     jsTrace.send( msg );
@@ -1395,10 +1416,10 @@ Object.extend(Object.extend(Jifty.Autocompleter.prototype, Ajax.Autocompleter.pr
   getUpdatedChoices: function() {
       var request = { path: this.url, actions: {} };
 
-      var a = $H();
+      var a = {}; //$H();
       a['moniker'] = 'autocomplete';
       a['class']   = 'Jifty::Action::Autocomplete';
-      a['fields']  = $H();
+      a['fields']  = {}; //$H();
       a['fields']['moniker']  = this.action.moniker;
       a['fields']['argument'] = Form.Element.getField(this.field);
       request['actions']['autocomplete'] = a;
@@ -1489,3 +1510,49 @@ if( !Object.prototype.hasOwnProperty ) {
         return true;
     }
 }
+
+function _sp_submit_form(elt, event, submit_to) {
+    var form = Form.Element.getForm(elt);
+    var elements = Form.getElements(form);
+
+    // Three things need to get merged -- hidden defaults, defaults
+    // from buttons, and form values.  Hence, we build up three lists
+    // and then merge them.
+    var hiddens = $H();
+    var buttons = $H();
+    var inputs = $H();
+    for (var i = 0; i < elements.length; i++) {
+        var e = elements[i];
+        var parsed = e.getAttribute("name").match(/^J:V-region-__page\.(.*)/);
+        var extras = Form.Element.buttonArguments(e);
+        if (extras.keys().length > 1) {
+            // Button with values
+            for (var j = 0; j < extras.keys().length; j++) {
+                if ( extras.keys()[j] == 'extend' ) continue;
+                // Might also have J:V mappings on it
+                parsed = extras.keys()[j].match(/^J:V-region-__page\.(.*)/);
+                if ((parsed != null) && (parsed.length == 2)) {
+                    buttons.set(parsed[1], extras.values()[j]);
+                } else if (extras.keys()[j].length > 0) {
+                    inputs.set(extras.keys()[j], extras.values()[j]);
+                }
+                
+            }
+        } else if ((parsed != null) && (parsed.length == 2)) {
+            // Hidden default
+            hiddens.set(parsed[1], $F(e));
+        } else if (e.name.length > 0) {
+            // Straight up values
+            inputs.set(e.name, $F(e));
+        }
+    }
+
+    var args = hiddens.merge(buttons.merge(inputs));
+
+    /* we want to feed a common object instead of a Hash to Jifty.update */ 
+    var args_object = {};
+    args.each( function( pair ) { args_object[pair.key] = pair.value; } );
+
+    if(event.ctrlKey||event.metaKey||event.altKey||event.shiftKey) return true;
+    return Jifty.update( {'continuation':{},'actions':null,'fragments':[{'mode':'Replace','args':args_object,'region':'__page','path': submit_to}]}, elt );
+}
